# Développement piloté par les tests (TDD) : Processus en plusieurs cycles

## Résumé
Le TDD, ou **Test-Driven Development**, est une méthode de développement où les tests unitaires sont écrits avant le code fonctionnel.

## Cycle très court pour ajouter une fonctionnalité :
1. **Rouge** : Écrire un test unitaire qui va échouer.
2. **Vert** : Écrire le code pour que le test réussisse.
3. **Refactor** : Améliorer le code tout en s'assurant que le test réussit toujours.

## Faits
🧪 Le TDD consiste à écrire des tests unitaires avant le code fonctionnel.  
✅ Le TDD assure que le code répond aux exigences et valide la pertinence des tests.  
🔒 Il offre une sécurité pour les futurs changements dans le code.  
💭 Il favorise la réflexion préliminaire sur le comportement souhaité du code.  
🏗️ Il peut conduire à un code mieux conçu et plus robuste.  
⏱️ Il réduit potentiellement le temps de débogage.


## Objectif : Créer un gestionnaire de dossier avec des fonctionnalités d'ajout, de suppression et de recherche de dossier.

### Cycle 1 : Ajouter un fichier

1. **Écrire un test**  
   Nous commençons par une fonction `ajouter_fichier` qui doit ajouter un fichier à une liste de dossier.
   
   Test :
   ```python
   def test_ajouter_fichier():
       dossier = []
       dossier = ajouter_fichier(dossier, "document.txt")
       assert "document.txt" in dossier
   ```

2. **Exécuter le test et le voir échouer**  
   La fonction `ajouter_fichier` n'existe pas encore, donc le test échoue.

3. **Écrire le code minimal**  
   Nous implémentons une fonction simple qui ajoute un fichier à une liste.
   ```python
   def ajouter_fichier(dossier, fichier):
       dossier.append(fichier)
       return dossier
   ```

4. **Exécuter le test et le voir réussir**  
   Le test passe. Nous avons implémenté l'ajout de fichier.

---

### Cycle 2 : Empêcher l'ajout de doublons

1. **Écrire un nouveau test**  
   Ajoutons une fonctionnalité : la fonction ne doit pas ajouter un fichier déjà présent dans la liste.

   Test :
   ```python
   def test_ajouter_fichier_sans_doublons():
       dossier = ["document.txt"]
       dossier = ajouter_fichier(dossier, "document.txt")
       assert dossier.count("document.txt") == 1
   ```

2. **Exécuter le test et le voir échouer**  
   Le test échoue car la fonction actuelle ajoute des doublons.

3. **Modifier le code**  
   Nous modifions la fonction pour vérifier si le fichier est déjà dans la liste avant de l'ajouter.
   ```python
   def ajouter_fichier(dossier, fichier):
       if fichier not in dossier:
           dossier.append(fichier)
       return dossier
   ```

4. **Exécuter le test et le voir réussir**  
   Le test passe. Nous avons maintenant une fonctionnalité qui empêche les doublons.

---

### Cycle 3 : Supprimer un fichier

1. **Écrire un test**  
   Ajoutons une nouvelle fonctionnalité : la possibilité de supprimer un fichier.

   Test :
   ```python
   def test_supprimer_fichier():
       dossier = ["document.txt", "image.jpg"]
       dossier = supprimer_fichier(dossier, "document.txt")
       assert "document.txt" not in dossier
   ```

2. **Exécuter le test et le voir échouer**  
   La fonction `supprimer_fichier` n'existe pas encore, donc le test échoue.

3. **Écrire le code minimal**  
   Nous implémentons une fonction simple pour supprimer un fichier de la liste.
   ```python
   def supprimer_fichier(dossier, fichier):
       dossier.remove(fichier)
       return dossier
   ```

4. **Exécuter le test et le voir réussir**  
   Le test passe. Nous avons ajouté la fonctionnalité de suppression de fichier.

---

### Cycle 4 : Gérer les erreurs de suppression (fichier non présent)

1. **Écrire un test**  
   Ajoutons une gestion d'erreurs : si le fichier à supprimer n'est pas présent, nous ne devons pas provoquer d'erreur.

   Test :
   ```python
   def test_supprimer_fichier_non_present():
       dossier = ["image.jpg"]
       dossier = supprimer_fichier(dossier, "document.txt")
       assert dossier == ["image.jpg"]
   ```

2. **Exécuter le test et le voir échouer**  
   Le test échoue car la fonction actuelle lève une exception si le fichier n'existe pas dans la liste.

3. **Modifier le code**  
   Nous modifions la fonction pour vérifier si le fichier est dans la liste avant de le supprimer.
   ```python
   def supprimer_fichier(dossier, fichier):
       if fichier in dossier:
           dossier.remove(fichier)
       return dossier
   ```

4. **Exécuter le test et le voir réussir**  
   Le test passe. La fonction gère maintenant correctement les tentatives de suppression de dossier non présents.

---

### Cycle 5 : Rechercher un fichier

1. **Écrire un test**  
   Nous ajoutons maintenant une fonctionnalité de recherche pour vérifier si un fichier est présent dans la liste.

   Test :
   ```python
   def test_rechercher_fichier():
       dossier = ["document.txt", "image.jpg"]
       assert rechercher_fichier(dossier, "image.jpg") is True
       assert rechercher_fichier(dossier, "audio.mp3") is False
   ```

2. **Exécuter le test et le voir échouer**  
   La fonction `rechercher_fichier` n'existe pas encore, donc le test échoue.

3. **Écrire le code minimal**  
   Nous implémentons une fonction de recherche qui retourne un booléen selon la présence du fichier.
   ```python
   def rechercher_fichier(dossier, fichier):
       return fichier in dossier
   ```

4. **Exécuter le test et le voir réussir**  
   Le test passe. Nous avons ajouté la fonctionnalité de recherche.

---

## Conclusion
À chaque cycle de développement, une nouvelle fonctionnalité est ajoutée à l'application (gestion des doublons, suppression, recherche) en suivant les principes du TDD. Ce processus montre comment on peut améliorer et enrichir progressivement un programme en écrivant d'abord les tests, puis le code correspondant.
```

Ce processus en 5 cycles présente un gestionnaire de dossier simple et illustre bien comment de nouvelles fonctionnalités nécessitent des modifications du code. Chaque cycle apporte une nouvelle fonctionnalité pratique avec des tests correspondants.